#!/usr/bin/env python
"""Example script demonstrating usage of the systemair_api package."""

import os
import time
from dotenv import load_dotenv

from systemair_api.auth.authenticator import SystemairAuthenticator
from systemair_api.api.systemair_api import SystemairAPI
from systemair_api.api.websocket_client import SystemairWebSocket
from systemair_api.models.ventilation_unit import VentilationUnit
from systemair_api.utils.constants import UserModes

# Dictionary to store ventilation units by device ID
ventilation_units = {}

def get_user_devices(api):
    """Get and initialize VentilationUnit objects for all user devices."""
    response = api.get_account_devices()
    if response and 'data' in response:
        devices = response['data'].get('GetAccountDevices', [])
        for device in devices:
            unit = VentilationUnit(device["identifier"], device["name"])
            ventilation_units[unit.identifier] = unit
    else:
        print("Failed to fetch user devices")

def on_websocket_message(data):
    """Handle incoming WebSocket messages."""
    if data["type"] == "SYSTEM_EVENT" and data["action"] == "DEVICE_STATUS_UPDATE":
        device_id = data["properties"]["id"]
        if device_id in ventilation_units:
            ventilation_units[device_id].update_from_websocket(data)
            ventilation_units[device_id].print_status()

def main():
    """Run the example application."""
    # Load environment variables
    load_dotenv()
    
    # Get credentials
    email = os.getenv('EMAIL')
    password = os.getenv('PASSWORD')

    if not email or not password:
        print("Error: Email or password not found in environment variables")
        return

    authenticator = SystemairAuthenticator(email, password)
    api = None
    websocket_client = None

    try:
        # Initial authentication
        print("Authenticating...")
        access_token = authenticator.authenticate()
        api = SystemairAPI(access_token)

        get_user_devices(api)
        if not ventilation_units:
            print("No devices found. Exiting.")
            return

        websocket_client = SystemairWebSocket(access_token, on_websocket_message)
        websocket_client.connect()
        device_ids = list(ventilation_units.keys())

        # Broadcast device statuses
        broadcast_result = api.broadcast_device_statuses(device_ids)
        print("Fetching initial status for all devices...")
        for device_id, unit in ventilation_units.items():
            try:
                device_status = api.fetch_device_status(device_id)
                if device_status:
                    unit.update_from_api(device_status)
                    unit.print_status()
                else:
                    print(f"Failed to fetch device status for {unit.name}")
            except Exception as e:
                print(f"Error fetching device status for {unit.name}: {e}")

        while True:
            print(f"\nCurrent token expiry: {authenticator.token_expiry}")

            # Check if token is still valid
            if not authenticator.is_token_valid():
                print("Token expired. Refreshing...")
                try:
                    access_token = authenticator.refresh_access_token()
                    print(f"Token refreshed. New expiry: {authenticator.token_expiry}")
                    api = SystemairAPI(access_token)
                    websocket_client.disconnect()
                    websocket_client = SystemairWebSocket(access_token, on_websocket_message)
                    websocket_client.connect()
                except Exception as e:
                    print(f"Failed to refresh token: {e}")
                    print("Re-authenticating...")
                    access_token = authenticator.authenticate()
                    api = SystemairAPI(access_token)
                    websocket_client.disconnect()
                    websocket_client = SystemairWebSocket(access_token, on_websocket_message)
                    websocket_client.connect()
                broadcast_result = api.broadcast_device_statuses(device_ids)

            # Fetch updated status for all devices
            print("\nRefreshing device statuses...")
            for device_id, unit in ventilation_units.items():
                try:
                    device_status = api.fetch_device_status(device_id)
                    if device_status:
                        unit.update_from_api(device_status)
                        unit.print_status()
                    else:
                        print(f"Failed to fetch device status for {unit.name}")
                except Exception as e:
                    print(f"Error fetching device status for {unit.name}: {e}")

            # Wait before next refresh
            print(f"\nWaiting 60 seconds before next refresh...")
            time.sleep(60)

    except KeyboardInterrupt:
        print("Program terminated by user.")
    except Exception as e:
        print(f"Unexpected error: {e}")
    finally:
        if websocket_client:
            websocket_client.disconnect()

if __name__ == "__main__":
    main()


"""Ventilation data constants and definitions."""

from typing import Dict, Any, Optional, Union, List

USER_MODES: Dict[int, Dict[str, str]] = {
    0: {
        "name": "Auto",
        "description": "Takes care of airflow levels automatically",
        "info": "AUTO mode is available to select if Demand Control or Week Schedule functions are configured and/or airflow type is selected \"External\". Demand Control shall control the airflow to ensure the best possible air quality with respect to RH and/or CO2 sensor readings."
    },
    1: {
        "name": "Manual",
        "description": "Set the airflow levels manually",
        "info": "The airflow levels are controlled manually. The unit can be set to run on one out of four available airflow levels: OFF/LOW/NORMAL/HIGH."
    },
    2: {
        "name": "Crowded",
        "description": "When you have extra guests",
        "info": "Crowded function increases the airflow to level HIGH and reduces the temperature setpoint to ensure a perfect indoor climate when there are guests. The delay is in hours."
    },
    3: {
        "name": "Refresh",
        "description": "Quick change of air",
        "info": "The airflow levels are increased to HIGH to replace the indoor air with fresh air in a short period of time. Function duration in minutes."
    },
    4: {
        "name": "Fireplace",
        "description": "Start your fire without smoke inside",
        "info": "The airflow levels are regulated so that a slight overpressure is created indoors (Supply air fan - NORMAL, Extract air fan - LOW) for better smoke extraction through the chimney. Function duration can be set in minutes."
    },
    5: {
        "name": "Away",
        "description": "Save energy while being away",
        "info": "To save energy when You are away from home for a short period of time, the airflow levels are reduced to LOW and the ECO mode function is activated. The function duration can be set in hours, default duration - 10 hours."
    },
    6: {
        "name": "Holiday",
        "description": "Saves energy while You are on Holidays",
        "info": "To save energy when You are on Holidays the airflow levels are reduced to LOW and the ECO mode is activated. The function duration can be set in Days, default duration - 16 Days."
    }
}

AIRFLOW_LEVELS: Dict[int, str] = {
    0: "Off",
    1: "Minimum",
    2: "Low",
    3: "Normal",
    4: "High",
    5: "Maximum",
    6: "Demand",
    7: "External"
}

class VentilationData:
    """Data definitions for ventilation units."""
    
    @staticmethod
    def get_user_mode_name(mode_id: int) -> str:
        """Get the name of a user mode by ID."""
        return USER_MODES.get(mode_id, {}).get("name", "Unknown")
    
    @staticmethod
    def get_airflow_level_name(level_id: int) -> str:
        """Get the name of an airflow level by ID."""
        return AIRFLOW_LEVELS.get(level_id, "Unknown")



#!/usr/bin/env python3
"""Fix imports in copied SystemAIR API files."""

import os
import re


def fix_imports(directory):
    """Fix imports in all Python files in the directory."""
    for root, _, files in os.walk(directory):
        for file in files:
            if file.endswith('.py'):
                file_path = os.path.join(root, file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Replace imports
                new_content = re.sub(
                    r'from systemair_api\.', 
                    r'from ....api.', 
                    content
                )
                
                # Prevent over-dotting in __init__.py files
                new_content = re.sub(
                    r'from ....api\.api\.', 
                    r'from ...api.', 
                    new_content
                )
                new_content = re.sub(
                    r'from ....api\.auth\.', 
                    r'from ...auth.', 
                    new_content
                )
                new_content = re.sub(
                    r'from ....api\.models\.', 
                    r'from ...models.', 
                    new_content
                )
                new_content = re.sub(
                    r'from ....api\.utils\.', 
                    r'from ...utils.', 
                    new_content
                )
                
                # Special case for __init__.py files
                if file == '__init__.py':
                    new_content = re.sub(
                        r'from ....api\.', 
                        r'from ..', 
                        new_content
                    )
                
                # Special case for main files
                if root.endswith('api'):
                    new_content = re.sub(
                        r'from ....api\.', 
                        r'from .', 
                        new_content
                    )
                
                # Write changes
                if new_content != content:
                    with open(file_path, 'w', encoding='utf-8') as f:
                        f.write(new_content)
                    print(f"Fixed imports in {file_path}")


if __name__ == "__main__":
    fix_imports('/Users/henningberge/PycharmProjects/SystemAIR-API/custom_components/systemair/api')


"""Exception classes for the SystemAIR API."""

from typing import Any, Dict, Optional


class SystemairError(Exception):
    """Base exception for all SystemAIR API related errors."""

    def __init__(self, message: str, *args: Any) -> None:
        """Initialize the exception.

        Args:
            message: The error message
            *args: Additional arguments to pass to the parent class
        """
        self.message = message
        super().__init__(message, *args)


class AuthenticationError(SystemairError):
    """Exception raised for authentication failures."""

    def __init__(self, message: str = "Authentication failed", *args: Any) -> None:
        super().__init__(message, *args)


class TokenRefreshError(AuthenticationError):
    """Exception raised when token refresh fails."""

    def __init__(
        self, message: str = "Failed to refresh access token", *args: Any
    ) -> None:
        super().__init__(message, *args)


class APIError(SystemairError):
    """Exception raised for all API related errors."""

    def __init__(
        self,
        message: str = "API request failed",
        status_code: Optional[int] = None,
        response_data: Optional[Dict[str, Any]] = None,
        *args: Any,
    ) -> None:
        """Initialize the exception.

        Args:
            message: The error message
            status_code: The HTTP status code
            response_data: The response data from the API
            *args: Additional arguments to pass to the parent class
        """
        self.status_code = status_code
        self.response_data = response_data
        if status_code:
            message = f"{message} (status code: {status_code})"
        super().__init__(message, *args)


class DeviceNotFoundError(APIError):
    """Exception raised when a device is not found."""

    def __init__(
        self, device_id: str, message: Optional[str] = None, *args: Any
    ) -> None:
        """Initialize the exception.

        Args:
            device_id: The ID of the device that was not found
            message: The error message
            *args: Additional arguments to pass to the parent class
        """
        self.device_id = device_id
        if message is None:
            message = f"Device with ID {device_id} not found"
        super().__init__(message, None, None, *args)


class WebSocketError(SystemairError):
    """Exception raised for WebSocket related errors."""

    def __init__(
        self, message: str = "WebSocket connection error", *args: Any
    ) -> None:
        super().__init__(message, *args)


class RateLimitError(APIError):
    """Exception raised when rate limits are exceeded."""

    def __init__(
        self,
        message: str = "Rate limit exceeded",
        retry_after: Optional[int] = None,
        *args: Any,
    ) -> None:
        """Initialize the exception.

        Args:
            message: The error message
            retry_after: The number of seconds to wait before retrying
            *args: Additional arguments to pass to the parent class
        """
        self.retry_after = retry_after
        if retry_after:
            message = f"{message}. Retry after {retry_after} seconds."
        super().__init__(message, 429, None, *args)


class ValidationError(SystemairError):
    """Exception raised for validation errors."""

    def __init__(
        self,
        message: str = "Validation error",
        field: Optional[str] = None,
        value: Optional[Any] = None,
        *args: Any,
    ) -> None:
        """Initialize the exception.

        Args:
            message: The error message
            field: The field that failed validation
            value: The value that failed validation
            *args: Additional arguments to pass to the parent class
        """
        self.field = field
        self.value = value
        if field and value is not None:
            message = f"{message}: {field}={value}"
        elif field:
            message = f"{message}: {field}"
        super().__init__(message, *args)


"""Constants for the Systemair API, including endpoints and user modes."""
from enum import IntEnum

CLIENT_ID = "iot-application"
REDIRECT_URI = "https://homesolutions.systemair.com"
AUTH_URL = "https://sso.systemair.com/auth/realms/iot/protocol/openid-connect/auth"
TOKEN_URL = "https://sso.systemair.com/auth/realms/iot/protocol/openid-connect/token"
GATEWAY_API_URL = "https://homesolutions.systemair.com/gateway/api"
REMOTE_API_URL = "https://homesolutions.systemair.com/gateway/remote-api/"


class UserModes(IntEnum):
    """User modes for ventilation units."""
    AUTO = 0
    MANUAL = 1
    CROWDED = 2
    REFRESH = 3
    FIREPLACE = 4
    AWAY = 5
    HOLIDAY = 6


class APIEndpoints:
    """API endpoints for Systemair API."""
    GATEWAY = GATEWAY_API_URL
    REMOTE = REMOTE_API_URL
    AUTH = AUTH_URL
    TOKEN = TOKEN_URL



#!/usr/bin/env python
"""Main entry point for the systemair_api package."""

import os
import time
import threading
from dotenv import load_dotenv

from systemair_api.auth.authenticator import SystemairAuthenticator
from systemair_api.api.systemair_api import SystemairAPI
from systemair_api.api.websocket_client import SystemairWebSocket
from systemair_api.models.ventilation_unit import VentilationUnit
from systemair_api.utils.constants import UserModes

def on_message(message: dict) -> None:
    """Handle incoming WebSocket messages.
    
    Args:
        message: The message received from the WebSocket
    """
    print("Received WebSocket message:", message)

def main() -> None:
    """Run the example application demonstrating the SystemAIR API."""
    # Load environment variables from .env file if it exists
    load_dotenv()
    
    # Get credentials from environment variables
    email = os.getenv("EMAIL")
    password = os.getenv("PASSWORD")
    
    if not email or not password:
        print("Please set EMAIL and PASSWORD environment variables")
        print("You can create a .env file with these variables")
        return
    
    # Authenticate
    authenticator = SystemairAuthenticator(email, password)
    access_token = authenticator.authenticate()
    
    if not access_token:
        print("Authentication failed")
        return
    
    print("Authentication successful, access token obtained")
    
    # Create API client
    api = SystemairAPI(access_token)
    
    # Get account devices
    devices_response = api.get_account_devices()
    if not devices_response or "data" not in devices_response:
        print("Failed to get devices")
        return
    
    devices = devices_response["data"]["GetAccountDevices"]
    if not devices:
        print("No devices found")
        return
    
    print(f"Found {len(devices)} devices:")
    ventilation_units = []
    
    for device in devices:
        print(f" - {device['name']} (ID: {device['identifier']})")
        ventilation_unit = VentilationUnit(device['identifier'], device['name'])
        ventilation_units.append(ventilation_unit)
    
    # Set up WebSocket client
    ws_client = SystemairWebSocket(access_token, on_message)
    ws_client.connect()
    
    try:
        # Get status for the first device
        if ventilation_units:
            unit = ventilation_units[0]
            
            # Get device status
            status_response = api.fetch_device_status(unit.identifier)
            if status_response:
                unit.update_from_api(status_response)
                print(f"\nCurrent Status for {unit.name}:")
                unit.print_status()
                
                # Broadcast to get updates via WebSocket
                api.broadcast_device_statuses([unit.identifier])
                
                # Example: Change user mode to Away
                print("\nChanging user mode to Away...")
                unit.set_user_mode(api, UserModes.AWAY)
                
                # Wait for a moment to receive WebSocket updates
                time.sleep(5)
                
                # Change user mode back to Auto
                print("\nChanging user mode back to Auto...")
                unit.set_user_mode(api, UserModes.AUTO)
                
                # Get updated status
                status_response = api.fetch_device_status(unit.identifier)
                if status_response:
                    unit.update_from_api(status_response)
                    print("\nUpdated Status:")
                    unit.print_status()
        
        # Keep the main thread alive for WebSocket messages
        print("\nListening for WebSocket messages. Press Ctrl+C to exit.")
        while True:
            time.sleep(1)
            
    except KeyboardInterrupt:
        print("\nExiting...")
    finally:
        # Disconnect WebSocket
        ws_client.disconnect()

if __name__ == "__main__":
    main()


"""SystemairAuthenticator - Authentication module for Systemair Home Solutions cloud."""

import uuid
import requests
import json
import base64
from typing import Dict, Optional, Any, Union, cast
from datetime import datetime, timedelta
from bs4 import BeautifulSoup, Tag
from systemair_api.utils.constants import APIEndpoints, CLIENT_ID, REDIRECT_URI
from systemair_api.utils.exceptions import AuthenticationError, TokenRefreshError

class SystemairAuthenticator:
    """Authentication handler for Systemair Home Solutions cloud.
    
    Manages the OAuth2 authentication flow, including:
    - Initial login with username/password
    - Token exchange
    - Token refresh
    - Token validation
    """
    
    def __init__(self, email: str, password: str) -> None:
        """Initialize the authenticator with user credentials.
        
        Args:
            email: User's email address
            password: User's password
        """
        self.email: str = email
        self.password: str = password
        self.session: requests.Session = requests.Session()
        self.access_token: Optional[str] = None
        self.refresh_token: Optional[str] = None
        self.token_expiry: Optional[datetime] = None

    def generate_state_parameter(self) -> str:
        """Generate a random state parameter for the OAuth flow.
        
        Returns:
            str: A random UUID as string
        """
        return str(uuid.uuid4())

    def construct_auth_url(self, state: str) -> str:
        """Construct the OAuth authorization URL.
        
        Args:
            state: State parameter to include in the URL
            
        Returns:
            str: Complete authorization URL
        """
        params = {
            "client_id": CLIENT_ID,
            "response_type": "code",
            "state": state,
            "redirect_uri": REDIRECT_URI,
            "scope": "openid"
        }
        query_string = "&".join([f"{key}={value}" for key, value in params.items()])
        return f"{APIEndpoints.AUTH}?{query_string}"

    def simulate_login(self, auth_url: str) -> str:
        """Simulate a browser login to obtain the authorization code.
        
        This method simulates the browser login process by:
        1. Fetching the login page
        2. Extracting the form and input fields
        3. Submitting the form with credentials
        4. Following redirects to obtain the authorization code
        
        Args:
            auth_url: The authorization URL to start the flow
            
        Returns:
            str: The authorization code if successful
            
        Raises:
            Exception: If login fails or authorization code cannot be obtained
        """
        # print(f"Fetching login page from: {auth_url}")
        response = self.session.get(auth_url)
        soup = BeautifulSoup(response.content, 'html.parser')

        form = soup.find('form')
        if not form:
            raise AuthenticationError('Login form not found')

        # Cast to Tag to ensure proper type handling
        form_tag = cast(Tag, form)
        action_url = form_tag['action']
        inputs = form_tag.find_all('input')

        form_data = {}
        for input_tag in inputs:
            # Cast each input element to Tag
            input_tag = cast(Tag, input_tag)
            if input_tag.get('name') == 'username':
                form_data[input_tag['name']] = self.email
            elif input_tag.get('name') == 'password':
                form_data[input_tag['name']] = self.password
            elif input_tag.get('name'):
                form_data[input_tag['name']] = input_tag.get('value', '')

        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:128.0) Gecko/20100101 Firefox/128.0',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/png,image/svg+xml,*/*;q=0.8',
            'Accept-Language': 'en-US,en;q=0.5',
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        # print("Submitting login form...")
        # Handle potential list type for action_url by ensuring it's a string
        if isinstance(action_url, list):
            action_url_str = action_url[0] if action_url else ''
        else:
            action_url_str = action_url
            
        response = self.session.post(action_url_str, data=form_data, headers=headers, allow_redirects=False)
        # print(f"Login form submission response status: {response.status_code} {response.reason}")

        if response.status_code == 302:
            redirect_url = response.headers.get('Location')
            # print("Redirect URL:", redirect_url)
            
            if redirect_url is None:
                raise AuthenticationError('Redirect URL not found in headers')

            response = self.session.get(redirect_url, allow_redirects=True)
            # print(f"Redirect follow-up response status: {response.status_code} {response.reason}")

            if response.status_code == 200:
                if 'code=' in response.url:
                    auth_code = response.url.split('code=')[1].split('&')[0]
                    # print("Authorization Code:", auth_code)
                    print("Authentication success")
                    return auth_code
                else:
                    raise AuthenticationError('Authorization code not found in final URL')
            else:
                raise AuthenticationError('Failed to follow redirect URL')
        else:
            raise AuthenticationError('Login failed or redirect did not occur')

    def exchange_code_for_token(self, auth_code: str) -> Dict[str, Any]:
        """Exchange an authorization code for access and refresh tokens.
        
        Args:
            auth_code: The authorization code obtained from login
            
        Returns:
            dict: Token response containing access_token, refresh_token, etc.
            
        Raises:
            requests.exceptions.HTTPError: If token exchange fails
        """
        data = {
            'grant_type': 'authorization_code',
            'code': auth_code,
            'redirect_uri': REDIRECT_URI,
            'client_id': CLIENT_ID,
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:128.0) Gecko/20100101 Firefox/128.0',
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded;charset=utf-8',
            'Origin': 'https://homesolutions.systemair.com',
            'Connection': 'keep-alive',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
            'TE': 'trailers',
        }

        response = requests.post(APIEndpoints.TOKEN, data=data, headers=headers)
        if response.status_code == 200:
            return cast(Dict[str, Any], response.json())
        else:
            print("Failed to exchange code for token:", response.content)
            response.raise_for_status()
            # This line is never reached but needed for mypy
            return {}

    def authenticate(self) -> str:
        """Perform the full authentication flow.
        
        This method orchestrates the complete authentication process:
        1. Generate state parameter
        2. Construct auth URL
        3. Simulate login to get authorization code
        4. Exchange code for tokens
        5. Extract token expiry time
        
        Returns:
            str: The access token if successful
            
        Raises:
            AuthenticationError: If authentication fails for any reason
        """
        state = self.generate_state_parameter()
        auth_url = self.construct_auth_url(state)
        auth_code = self.simulate_login(auth_url)
        token_response = self.exchange_code_for_token(auth_code)
        self.access_token = token_response.get('access_token')
        self.refresh_token = token_response.get('refresh_token')
        
        if not self.access_token:
            raise AuthenticationError('No access token found in response')
            
        self.token_expiry = self.get_token_expiry(self.access_token)
        return str(self.access_token)

    def refresh_access_token(self) -> str:
        """Refresh the access token using the refresh token.
        
        Returns:
            str: The new access token if successful
            
        Raises:
            TokenRefreshError: If refresh fails or no refresh token is available
        """
        if not self.refresh_token:
            raise TokenRefreshError("No refresh token available. Please authenticate first.")

        data = {
            'grant_type': 'refresh_token',
            'client_id': CLIENT_ID,
            'refresh_token': self.refresh_token,
            'redirect_uri': REDIRECT_URI,
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:128.0) Gecko/20100101 Firefox/128.0',
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded;charset=utf-8',
            'Origin': 'https://homesolutions.systemair.com',
            'Connection': 'keep-alive',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
        }

        response = requests.post(APIEndpoints.TOKEN, data=data, headers=headers)
        if response.status_code == 200:
            token_data = response.json()
            self.access_token = token_data.get('access_token')
            self.refresh_token = token_data.get('refresh_token')  # Update refresh token if provided
            
            if not self.access_token:
                raise TokenRefreshError('No access token found in refresh response')
                
            self.token_expiry = self.get_token_expiry(self.access_token)
            return str(self.access_token)
        else:
            raise TokenRefreshError(f"Failed to refresh token: {response.text}")

    def get_token_expiry(self, token: str) -> Optional[datetime]:
        """Decode the JWT and extract the expiry time.
        
        Args:
            token: JWT token to decode
            
        Returns:
            datetime: Token expiry time as datetime object
            
        Raises:
            ValueError: If expiry time cannot be found in the token
        """
        try:
            # Split the token and get the payload part (second part)
            payload = token.split('.')[1]

            # Add padding if necessary
            payload += '=' * ((4 - len(payload) % 4) % 4)

            # Decode the Base64 string
            decoded_payload = base64.b64decode(payload)

            # Parse the JSON
            token_data = json.loads(decoded_payload)

            # Extract the expiry time
            exp_timestamp = token_data.get('exp')

            if exp_timestamp:
                return datetime.fromtimestamp(exp_timestamp)
            else:
                raise ValueError("No expiry time found in token")
        except Exception as e:
            print(f"Error decoding token: {e}")
            return None

    def is_token_valid(self) -> bool:
        """Check if the current token is still valid.
        
        Returns:
            bool: True if token is valid, False otherwise
        """
        if not self.token_expiry:
            return False

        # Consider the token invalid if it's about to expire in the next 30 seconds
        return datetime.now() + timedelta(seconds=30) < self.token_expiry



"""Configuration file for the Sphinx documentation builder."""

import os
import sys

# Add project root to the path so that autodoc can find it
sys.path.insert(0, os.path.abspath('../..'))

# -- Project information -----------------------------------------------------
project = 'SystemAIR-API'
copyright = '2025, Henning Berge'
author = 'Henning Berge'

# The full version, including alpha/beta/rc tags
import importlib.metadata
try:
    release = importlib.metadata.version('systemair_api')
except importlib.metadata.PackageNotFoundError:
    release = '0.1.0'

# -- General configuration ---------------------------------------------------
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.viewcode',
    'sphinx.ext.napoleon',
    'sphinx.ext.intersphinx',
    'sphinx_autodoc_typehints',
]

intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'requests': ('https://requests.readthedocs.io/en/latest/', None),
}

templates_path = ['_templates']
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------
html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']

# -- Napoleon settings -------------------------------------------------------
napoleon_google_docstring = True
napoleon_numpy_docstring = False
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True

# -- autodoc settings --------------------------------------------------------
autodoc_default_options = {
    'members': True,
    'member-order': 'bysource',
    'special-members': '__init__',
    'undoc-members': True,
    'exclude-members': '__weakref__, __str__, __repr__',
}

# -- TypeHints settings ------------------------------------------------------
typehints_fully_qualified = False
always_document_param_types = True


"""Ventilation unit model."""

from datetime import datetime
from typing import Dict, List, Optional, Any, Union, Tuple, Set, cast

from systemair_api.models.ventilation_data import USER_MODES, VentilationData
from systemair_api.utils.constants import UserModes
from systemair_api.utils.register_constants import RegisterConstants
from systemair_api.api.systemair_api import SystemairAPI


class VentilationUnit:
    """Model representing a Systemair ventilation unit."""
    
    def __init__(self, identifier: str, name: str) -> None:
        """Initialize a ventilation unit with a unique identifier and name."""
        self.identifier: str = identifier
        self.name: str = name
        self.model: Optional[str] = None
        self.active_alarms: bool = False
        self.airflow: Optional[int] = None
        self.connectivity: List[str] = []
        self.filter_expiration: Optional[int] = None
        self.serial_number: Optional[str] = None
        self.temperature: Optional[float] = None
        self.user_mode: Optional[int] = None
        self.air_quality: Optional[int] = None
        self.humidity: Optional[int] = None
        self.co2: Optional[int] = None
        self.update_in_progress: bool = False
        self.configuration_wizard_active: bool = False
        self.user_mode_remaining_time: Optional[int] = None
        self.user_mode_times: Dict[str, Optional[int]] = {
            "holiday": None,  # REG_MAINBOARD_USERMODE_HOLIDAY_TIME
            "away": None,     # REG_MAINBOARD_USERMODE_AWAY_TIME
            "fireplace": None, # REG_MAINBOARD_USERMODE_FIREPLACE_TIME
            "refresh": None,  # REG_MAINBOARD_USERMODE_REFRESH_TIME
            "crowded": None,  # REG_MAINBOARD_USERMODE_CROWDED_TIME
        }
        self.temperatures: Dict[str, Optional[float]] = {
            "oat": None,  # Outdoor Air Temperature
            "sat": None,  # Supply Air Temperature
            "setpoint": None
        }
        self.versions: List[Dict[str, str]] = []

        # Attributes from API data
        self.eco_mode: Optional[int] = None
        self.locked_user: Optional[int] = None
        self.alarm_type_a: Optional[int] = None
        self.alarm_type_b: Optional[int] = None
        self.alarm_type_c: Optional[int] = None
        self.suw_required: Optional[int] = None
        self.reheater_type: Optional[int] = None
        self.active_functions: Dict[str, bool] = {
            'cooling': False,
            'free_cooling': False,
            'heating': False,
            'defrosting': False,
            'heat_recovery': False,
            'cooling_recovery': False,
            'moisture_transfer': False,
            'secondary_air': False,
            'vacuum_cleaner': False,
            'cooker_hood': False,
            'user_lock': False,
            'eco_mode': False,
            'heater_cool_down': False,
            'pressure_guard': False,
            'cdi_1': False,
            'cdi_2': False,
            'cdi_3': False
        }

    def update_from_api(self, api_data: Dict[str, Any]) -> None:
        """Update the ventilation unit with data from the API."""
        if 'data' in api_data and 'GetView' in api_data['data']:
            children = api_data['data']['GetView']['children']
            for child in children:
                if 'properties' in child and 'dataItem' in child['properties']:
                    data_item = child['properties']['dataItem']
                    self._update_attribute(data_item)

    def _update_attribute(self, data_item: Dict[str, Any]) -> None:
        """Update a specific attribute based on register data."""
        register_id = data_item['id']
        value = data_item['value']

        if register_id == RegisterConstants.REG_MAINBOARD_USERMODE_MODE_HMI:
            self.user_mode = value
        elif register_id == RegisterConstants.REG_MAINBOARD_SPEED_INDICATION_APP:
            self.airflow = value
        elif register_id == RegisterConstants.REG_MAINBOARD_TC_SP:
            self.temperatures['setpoint'] = value / 10.0
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_REMAINING_TIME_L:
            self.user_mode_remaining_time = value
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_HOLIDAY_TIME:
            self.user_mode_times['holiday'] = value
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_AWAY_TIME:
            self.user_mode_times['away'] = value
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_FIREPLACE_TIME:
            self.user_mode_times['fireplace'] = value
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_REFRESH_TIME:
            self.user_mode_times['refresh'] = value
        elif register_id == RegisterConstants.REG_MAINBOARD_USERMODE_CROWDED_TIME:
            self.user_mode_times['crowded'] = value
        elif register_id == RegisterConstants.REG_MAINBOARD_IAQ_LEVEL:
            self.air_quality = value
        elif register_id == RegisterConstants.REG_MAINBOARD_SENSOR_OAT:
            self.temperatures['oat'] = value / 10.0
        elif register_id == RegisterConstants.REG_MAINBOARD_ECO_MODE_ON_OFF:
            self.eco_mode = value
        elif register_id == RegisterConstants.REG_MAINBOARD_LOCKED_USER:
            self.locked_user = value
        elif register_id == RegisterConstants.REG_MAINBOARD_ALARM_TYPE_A:
            self.alarm_type_a = value
        elif register_id == RegisterConstants.REG_MAINBOARD_ALARM_TYPE_B:
            self.alarm_type_b = value
        elif register_id == RegisterConstants.REG_MAINBOARD_ALARM_TYPE_C:
            self.alarm_type_c = value
        elif register_id == RegisterConstants.REG_MAINBOARD_SUW_REQUIRED:
            self.suw_required = value
        elif register_id == RegisterConstants.REG_MAINBOARD_UNIT_CONFIG_REHEATER_TYPE:
            self.reheater_type = value
        elif register_id in range(RegisterConstants.REG_MAINBOARD_FUNCTION_ACTIVE_COOLING,
                                  RegisterConstants.REG_MAINBOARD_FUNCTION_ACTIVE_CDI_3 + 1):
            function_name = RegisterConstants.get_register_name(register_id).split('_')[-1].lower()
            self.active_functions[function_name] = value

    def update_from_websocket(self, ws_data: Dict[str, Any]) -> None:
        """Update the ventilation unit with data from a WebSocket message."""
        properties = ws_data

        # Extract property data if it's nested in 'properties'
        if "properties" in ws_data:
            properties = ws_data.get("properties", {})

        # Update basic properties
        if "model" in properties:
            self.model = properties.get("model")
        if "activeAlarms" in properties:
            self.active_alarms = properties.get("activeAlarms")
        if "airflow" in properties:
            self.airflow = properties.get("airflow")
        if "connectivity" in properties:
            self.connectivity = properties.get("connectivity")
        if "filterExpiration" in properties:
            self.filter_expiration = properties.get("filterExpiration")
        if "serialNumber" in properties:
            self.serial_number = properties.get("serialNumber")
        if "temperature" in properties:
            self.temperature = properties.get("temperature")
        if "userMode" in properties:
            self.user_mode = properties.get("userMode")
        if "airQuality" in properties:
            self.air_quality = properties.get("airQuality")
        if "humidity" in properties:
            self.humidity = properties.get("humidity")
        if "co2" in properties:
            self.co2 = properties.get("co2")

        # Update nested properties
        update_info = properties.get("update", {})
        if update_info and "inProgress" in update_info:
            self.update_in_progress = update_info.get("inProgress")

        config_wizard = properties.get("configurationWizard", {})
        if config_wizard and "active" in config_wizard:
            self.configuration_wizard_active = config_wizard.get("active")

        # Update temperature data
        temps = properties.get("temperatures", {})
        if temps:
            # Only update specific temperature keys that exist
            for key, value in temps.items():
                if key in self.temperatures:
                    self.temperatures[key] = value

        # Update version data
        versions = properties.get("versions", [])
        if versions:
            self.versions = versions

    def __str__(self) -> str:
        """String representation of the ventilation unit."""
        return f"VentilationUnit: {self.name} (ID: {self.identifier})"

    @property
    def user_mode_name(self) -> str:
        """Get the name of the current user mode."""
        if self.user_mode is None:
            return "Unknown"
        return VentilationData.get_user_mode_name(self.user_mode)
        
    def get_fan_speed(self) -> int:
        """Get the fan speed as a percentage (0-100)."""
        return self.airflow or 0
        
    def get_filter_alarm(self) -> bool:
        """Check if there's a filter alarm active."""
        # Placeholder implementation - will need to be updated based on how filter alarms are detected
        return self.active_alarms
        
    def get_status(self) -> Dict[str, Any]:
        """Get the current status of the ventilation unit as a dictionary."""
        return {
            "name": self.name,
            "model": self.model,
            "active_alarms": self.active_alarms,
            "airflow": self.airflow,
            "connectivity": self.connectivity,
            "filter_expiration": self.filter_expiration,
            "serial_number": self.serial_number,
            "temperature": self.temperature,
            "user_mode": self.user_mode,
            "user_mode_name": self.user_mode_name,
            "air_quality": self.air_quality,
            "humidity": self.humidity,
            "co2": self.co2,
            "update_in_progress": self.update_in_progress,
            "configuration_wizard_active": self.configuration_wizard_active,
            "temperatures": self.temperatures,
            "versions": self.versions,
            "eco_mode": self.eco_mode,
            "locked_user": self.locked_user,
            "alarm_type_a": self.alarm_type_a,
            "alarm_type_b": self.alarm_type_b,
            "alarm_type_c": self.alarm_type_c,
            "suw_required": self.suw_required,
            "reheater_type": self.reheater_type,
            "active_functions": self.active_functions,
            "user_mode_remaining_time": self.user_mode_remaining_time,
            "user_mode_times": self.user_mode_times
        }

    def print_status(self) -> None:
        """Print the current status of the ventilation unit."""
        print(f"\n{datetime.now()} - Status for {self.name}:")
        status = self.get_status()
        for key, value in status.items():
            if key not in ["active_functions", "versions", "connectivity"]:
                print(f"{key.replace('_', ' ').title()}: {value}")

        print("Temperatures:")
        for temp_key, temp_value in status["temperatures"].items():
            print(f"  - {temp_key.upper()}: {temp_value}")

        print("Versions:")
        for version in status["versions"]:
            print(f"  - {version['type'].upper()}: {version['version']}")

        print("Connectivity:", status["connectivity"])

        print("Active Functions:")
        for function, is_active in status["active_functions"].items():
            if is_active:
                print(f"  - {function.replace('_', ' ').title()}")

    def set_value(self, api: SystemairAPI, key: int, value: Union[int, float, str], noprint: bool = False) -> Optional[bool]:
        """Set a register value for the ventilation unit.
        
        Args:
            api: The SystemairAPI instance to use for communication
            key: The register key to set
            value: The value to set
            noprint: Whether to suppress print output
            
        Returns:
            bool: True if successful, False otherwise
        """
        result = api.write_data_item(
            self.identifier,
            key,
            value
        )
        if not noprint:
            if result and result.get('data', {}).get('WriteDataItems'):
                print(f"Value for {RegisterConstants.get_register_name_by_number(key)} set to {value}")
            else:
                print(f"Failed to set user mode for {self.name}")
            return None
        else:
            return bool(result and result.get('data', {}).get('WriteDataItems'))

    def set_user_mode(self, api: SystemairAPI, mode_value: int, time_minutes: Optional[int] = None) -> None:
        """Set the user mode for the ventilation unit.
        
        Args:
            api: The SystemairAPI instance to use for communication
            mode_value: The mode value to set (use UserModes enum)
            time_minutes: Optional time duration in minutes for timed modes
                (Refresh, Crowded, Fireplace, Away, Holiday)
            
        Returns:
            None
        """
        # Map mode values to their time registers and determine if we need to set time
        mode_to_time_register = {
            UserModes.HOLIDAY: RegisterConstants.REG_MAINBOARD_USERMODE_HOLIDAY_TIME,
            UserModes.AWAY: RegisterConstants.REG_MAINBOARD_USERMODE_AWAY_TIME,
            UserModes.FIREPLACE: RegisterConstants.REG_MAINBOARD_USERMODE_FIREPLACE_TIME,
            UserModes.REFRESH: RegisterConstants.REG_MAINBOARD_USERMODE_REFRESH_TIME,
            UserModes.CROWDED: RegisterConstants.REG_MAINBOARD_USERMODE_CROWDED_TIME,
        }
        
        # Set the time value first if provided and this is a timed mode
        time_register = mode_to_time_register.get(mode_value)
        if time_minutes is not None and time_register is not None:
            # Convert minutes to the units expected by each specific register
            api_time_value = self._convert_minutes_to_api_units(mode_value, time_minutes)
            self.set_value(api, time_register, api_time_value, True)
            # Update local cache (keep in minutes for internal use)
            mode_key = self.get_mode_name_for_key(mode_value)
            if mode_key and hasattr(self, "user_mode_times") and mode_key in self.user_mode_times:
                self.user_mode_times[mode_key] = time_minutes
        
        # Then set the mode
        if self.set_value(api, RegisterConstants.REG_MAINBOARD_USERMODE_HMI_CHANGE_REQUEST, mode_value + 1, True):
            # self.user_mode = mode_value
            print(f"User mode set to {USER_MODES.get(mode_value, {}).get('name', 'Unknown')} for {self.name}")
        else:
            print(f"Failed to set user mode for {self.name}")

    def _convert_minutes_to_api_units(self, mode_value: int, time_minutes: int) -> int:
        """Convert time in minutes to the units expected by the API for each mode.
        
        API expects:
        - HOLIDAY: days (REG_MAINBOARD_USERMODE_HOLIDAY_TIME = 251)
        - AWAY: hours (REG_MAINBOARD_USERMODE_AWAY_TIME = 252) 
        - FIREPLACE: minutes (REG_MAINBOARD_USERMODE_FIREPLACE_TIME = 253)
        - REFRESH: minutes (REG_MAINBOARD_USERMODE_REFRESH_TIME = 254)
        - CROWDED: hours (REG_MAINBOARD_USERMODE_CROWDED_TIME = 255)
        """
        if mode_value == UserModes.HOLIDAY:
            return max(1, time_minutes // (24 * 60))  # Convert to days, minimum 1
        elif mode_value in [UserModes.AWAY, UserModes.CROWDED]:
            return max(1, time_minutes // 60)  # Convert to hours, minimum 1
        elif mode_value in [UserModes.FIREPLACE, UserModes.REFRESH]:
            return time_minutes  # Already in minutes
        else:
            return time_minutes  # Fallback
            
    def get_mode_name_for_key(self, mode_value: int) -> str:
        """Map numeric mode value to string mode key.
        
        Args:
            mode_value: The numeric mode value from UserModes enum
            
        Returns:
            str: The string key name (e.g., 'holiday', 'away', etc.)
        """
        mode_map = {
            UserModes.HOLIDAY: "holiday",
            UserModes.AWAY: "away",
            UserModes.FIREPLACE: "fireplace",
            UserModes.REFRESH: "refresh",
            UserModes.CROWDED: "crowded"
        }
        return mode_map.get(mode_value, "")
            
    def set_temperature(self, api: SystemairAPI, temperature: int) -> None:
        """Set the temperature setpoint for the ventilation unit.
        
        Args:
            api: The SystemairAPI instance to use for communication
            temperature: Temperature in tenths of degrees (e.g., 210 for 21.0°C)
            
        Returns:
            None
        """
        if self.set_value(api, RegisterConstants.REG_MAINBOARD_TC_SP, temperature, True):
            print(f"Temperature set to {temperature/10.0:.1f}°C for {self.name}")
        else:
            print(f"Failed to set temperature for {self.name}")
            
    def set_user_mode_time(self, api: SystemairAPI, mode: str, time_value: int) -> None:
        """Set the time duration for a specific user mode.
        
        Args:
            api: The SystemairAPI instance to use for communication
            mode: The mode to set time for ('holiday', 'away', 'fireplace', 'refresh', 'crowded')
            time_value: The time duration in minutes
            
        Returns:
            None
        """
        # Map mode names to their respective registers and mode values
        mode_info = {
            'holiday': (RegisterConstants.REG_MAINBOARD_USERMODE_HOLIDAY_TIME, UserModes.HOLIDAY),
            'away': (RegisterConstants.REG_MAINBOARD_USERMODE_AWAY_TIME, UserModes.AWAY),
            'fireplace': (RegisterConstants.REG_MAINBOARD_USERMODE_FIREPLACE_TIME, UserModes.FIREPLACE),
            'refresh': (RegisterConstants.REG_MAINBOARD_USERMODE_REFRESH_TIME, UserModes.REFRESH),
            'crowded': (RegisterConstants.REG_MAINBOARD_USERMODE_CROWDED_TIME, UserModes.CROWDED),
        }
        
        if mode not in mode_info:
            print(f"Invalid mode: {mode}. Must be one of {list(mode_info.keys())}")
            return
            
        register, mode_value = mode_info[mode]
        
        # Convert minutes to the units expected by the API for this specific mode
        api_time_value = self._convert_minutes_to_api_units(mode_value, time_value)
        
        if self.set_value(api, register, api_time_value, True):
            # Show the user-friendly units in the message
            unit_name = "days" if mode == 'holiday' else ("hours" if mode in ['away', 'crowded'] else "minutes")
            print(f"{mode.capitalize()} mode time set to {api_time_value} {unit_name} for {self.name}")
            # Update our local cache (keep in minutes for internal consistency)
            self.user_mode_times[mode] = time_value
        else:
            print(f"Failed to set {mode} mode time for {self.name}")



#!/usr/bin/env python3
"""
Debug script for testing SystemAIR-API connection and data retrieval.

This standalone script helps diagnose connection issues by:
- Authenticating with the Systemair Home Solutions API
- Retrieving account devices
- Fetching device status data
- Creating VentilationUnit objects and printing their status

Usage:
    python debug_api.py

The script requires credentials in environment variables:
- EMAIL: Your Systemair account email
- PASSWORD: Your Systemair account password

Or you can use a .env file in the same directory.
"""

import os
import sys
import logging
import json
import traceback
from pprint import pprint
from pathlib import Path
from dotenv import load_dotenv

# Add the project root to the Python path
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

# Set up logging
script_dir = Path(__file__).parent.absolute()
log_file = script_dir / "systemair_debug.log"
print(f"Attempting to write log file to: {log_file}")
print(f"Current working directory: {os.getcwd()}")
print(f"Script directory: {script_dir}")

# Check if we can write to the directory
try:
    test_file = script_dir / "test_write_permission.tmp"
    with open(test_file, 'w') as f:
        f.write("test")
    os.remove(test_file)
    print(f"Directory is writable: {script_dir}")
except Exception as e:
    print(f"Directory is NOT writable: {script_dir}, Error: {str(e)}")

try:
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        handlers=[
            logging.FileHandler(str(log_file), mode='w'),
            logging.StreamHandler(sys.stdout)
        ]
    )
    print(f"Log file created successfully at: {log_file}")
except Exception as e:
    print(f"ERROR creating log file: {str(e)}")
    # Fallback to console-only logging
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        handlers=[
            logging.StreamHandler(sys.stdout)
        ]
    )

logger = logging.getLogger('SystemairDebug')

# Import the SystemAIR-API modules
from systemair_api.auth.authenticator import SystemairAuthenticator
from systemair_api.api.systemair_api import SystemairAPI
from systemair_api.models.ventilation_unit import VentilationUnit
from systemair_api.utils.exceptions import AuthenticationError, APIError, DeviceNotFoundError

def main():
    """Main function to test API connection and data retrieval."""
    # Direct output to both stdout and file
    log_file = open("debug_api.log", "w")
    
    def log(message):
        """Log to both stdout and file."""
        print(message)
        log_file.write(message + "\n")
        log_file.flush()  # Force writing to file
    
    log("Starting SystemAIR API debug script...")
    logger.info("Starting debug script")
    
    try:
        # Load environment variables from .env file
        load_dotenv()
        
        # Get credentials
        email = os.environ.get('EMAIL')
        password = os.environ.get('PASSWORD')
        
        if not email or not password:
            log("ERROR: Email or password not found in environment variables.")
            log("Please create a .env file with EMAIL and PASSWORD variables.")
            logger.error("Missing EMAIL or PASSWORD environment variables")
            return False
        
        log("Using email: " + email)
        
        try:
            # Step 1: Authenticate
            log("\n--- AUTHENTICATION ---")
            logger.info("Authenticating with Systemair API")
            auth = SystemairAuthenticator(email, password)
            access_token = auth.authenticate()
            logger.info(f"Authentication successful, token expires: {auth.token_expiry}")
            log("Authentication successful!")
            log(f"Token expires: {auth.token_expiry}")
            
            # Step 2: Create API client
            log("\n--- API CLIENT ---")
            logger.info("Creating API client")
            api = SystemairAPI(access_token)
            
            # Step 3: Get account devices
            log("\n--- FETCHING DEVICES ---")
            logger.info("Fetching account devices")
            devices_data = api.get_account_devices()
            
            # Log raw API response
            logger.debug(f"Raw API response: {json.dumps(devices_data, indent=2)}")
            
            # Check response structure
            if 'data' not in devices_data or 'GetAccountDevices' not in devices_data['data']:
                log("ERROR: Unexpected API response structure")
                logger.error("Unexpected API response structure")
                log(f"Response: {devices_data}")
                return False
            
            # Get devices
            devices = devices_data['data']['GetAccountDevices']
            device_count = len(devices)
            logger.info(f"Found {device_count} devices")
            log(f"Found {device_count} devices")
            
            if device_count == 0:
                log("WARNING: No devices found in account")
                logger.warning("No devices found in account")
                return False
            
            # Step 4: Process each device
            for i, device in enumerate(devices, 1):
                device_id = device['identifier']
                device_name = device['name']
                
                log(f"\n--- DEVICE {i}/{device_count}: {device_name} (ID: {device_id}) ---")
                logger.info(f"Processing device: {device_name} (ID: {device_id})")
                
                try:
                    # Fetch device status
                    log("Fetching device status...")
                    logger.info(f"Fetching status for device: {device_id}")
                    status_data = api.fetch_device_status(device_id)
                    
                    # Log raw status data
                    logger.debug(f"Raw device status: {json.dumps(status_data, indent=2)}")
                    
                    # Check response structure
                    if 'data' not in status_data or 'GetView' not in status_data['data']:
                        log(f"ERROR: Unexpected status response structure for device {device_id}")
                        logger.error(f"Unexpected status response structure for device {device_id}")
                        continue
                    
                    # Count data items
                    children = status_data['data']['GetView']['children']
                    data_items = [
                        child['properties']['dataItem'] 
                        for child in children 
                        if 'properties' in child and 'dataItem' in child['properties']
                    ]
                    logger.info(f"Found {len(data_items)} data items for device {device_id}")
                    log(f"Found {len(data_items)} data registers in device response")
                    
                    # Sample of data items (first 5)
                    if data_items:
                        logger.info("Sample data items:")
                        log("\nSample data registers:")
                        for item in data_items[:5]:
                            register_id = item['id']
                            value = item['value']
                            logger.info(f"  Register ID: {register_id}, Value: {value}")
                            log(f"  Register ID: {register_id}, Value: {value}")
                    
                    # Create ventilation unit
                    unit = VentilationUnit(device_id, device_name)
                    unit.update_from_api(status_data)
                    
                    # Print unit status
                    log("\nVentilation Unit Status:")
                    unit.print_status()
                    
                    logger.info(f"Successfully processed device {device_id}")
                    
                except DeviceNotFoundError as e:
                    log(f"ERROR: Device not found: {str(e)}")
                    logger.error(f"Device not found: {str(e)}")
                except APIError as e:
                    log(f"ERROR: API error: {str(e)}")
                    logger.error(f"API error: {str(e)}")
                except Exception as e:
                    log(f"ERROR: Unexpected error: {str(e)}")
                    logger.error(f"Unexpected error: {str(e)}", exc_info=True)
                    traceback.print_exc(file=log_file)
            
            # Test broadcasting device statuses
            log("\n--- BROADCASTING DEVICE STATUSES ---")
            logger.info("Testing broadcast device statuses")
            device_ids = [device['identifier'] for device in devices]
            log(f"Broadcasting status request for {len(device_ids)} devices...")
            broadcast_result = api.broadcast_device_statuses(device_ids)
            logger.debug(f"Broadcast result: {broadcast_result}")
            
            if broadcast_result and broadcast_result.get('data', {}).get('BroadcastDeviceStatuses') is True:
                log("Broadcast successful!")
            else:
                log("Broadcast response: " + str(broadcast_result))
            
            log("\nTest completed successfully!")
            logger.info("Debug script completed successfully")
            return True
            
        except AuthenticationError as e:
            log(f"ERROR: Authentication failed: {str(e)}")
            logger.error(f"Authentication failed: {str(e)}")
        except APIError as e:
            log(f"ERROR: API error: {str(e)}")
            logger.error(f"API error: {str(e)}")
        except Exception as e:
            log(f"ERROR: Unexpected error: {str(e)}")
            logger.error(f"Unexpected error: {str(e)}", exc_info=True)
            traceback.print_exc(file=log_file)
    
    except Exception as e:
        log(f"CRITICAL ERROR: {str(e)}")
        logger.critical(f"Unexpected error: {str(e)}", exc_info=True)
        traceback.print_exc(file=log_file)
    
    finally:
        log_file.close()
        log("Log files written to debug_api.log and systemair_debug.log")
        
    return False

if __name__ == "__main__":
    success = main()
    sys.exit(0 if success else 1)


"""Register constants for Systemair ventilation units."""

class RegisterConstants:
    """Register constants used for reading and writing data to ventilation units."""
    REG_IAM_HEARTBEAT = 0
    REG_IAM_UPTIME = 1
    REG_IAM_ACCESS_TOKEN = 2
    REG_IAM_CPU_IDENTIFIER = 3
    REG_IAM_PU_UPDATE_SOURCE = 4
    REG_IAM_UPDATE_PROCESS_STATE = 5
    REG_IAM_UPDATE_PROCESS_CURRENT_FILE_TRANSFER_PERCENTAGE = 6
    REG_IAM_UPDATE_PROCESS_CURRENT_FILE = 7
    REG_IAM_CFG_STORAGE_FILENAME_1 = 8
    REG_IAM_CFG_STORAGE_STATUS = 9
    REG_IAM_CFG_STORAGE_TRANSFER_STATE = 10
    REG_IAM_CFG_STORAGE_TRANSFER_PROGRESS = 11
    REG_IAM_CFG_STORAGE_TRANSFER_ERROR_CODE = 12
    REG_IAM_CFG_STORAGE_CTRL = 13
    REG_MAINBOARD_PASSWD_PC_UNLOCKED = 14
    REG_MAINBOARD_PASSWD_PC_SETTINGS = 15
    REG_MAINBOARD_FACTORY_RESET = 16
    REG_MAINBOARD_SET_USER_SAFE_CONFIG = 17
    REG_MAINBOARD_ACTIVATE_USER_SAFE_CONFIG = 18
    REG_MAINBOARD_USER_SAFE_CONFIG_VALID = 19
    REG_MAINBOARD_FILTER_PERIOD = 20
    REG_MAINBOARD_FILTER_PERIOD_SET = 21
    REG_MAINBOARD_FILTER_REMAINING_TIME_L = 22
    REG_MAINBOARD_FILTER_KIT_INDEX = 23
    REG_MAINBOARD_TIME_YEAR = 24
    REG_MAINBOARD_TIME_AUTO_SUM_WIN = 25
    REG_MAINBOARD_HOUR_FORMAT = 26
    REG_MAINBOARD_SYSTEM_UNIT_MODEL_TYPE = 27
    REG_MAINBOARD_SYSTEM_UNIT_TEMPERATURE = 28
    REG_MAINBOARD_USERMODE_MODE_HMI = 29 # Ventilation mode
    REG_MAINBOARD_USERMODE_HMI_CHANGE_REQUEST = 30 # Set ventilation mode ACTION
    REG_MAINBOARD_SPEED_INDICATION_APP = 31 # Air speed
    REG_MAINBOARD_TC_SP = 32 # Target setpoint supply air temp
    REG_MAINBOARD_IAQ_LEVEL = 33
    REG_MAINBOARD_ECO_MODE_ON_OFF = 34
    REG_MAINBOARD_LOCKED_USER = 35
    REG_MAINBOARD_LOCKED_FILTER = 36
    REG_MAINBOARD_LOCKED_WEEK_SCHEDULE = 37
    REG_MAINBOARD_USERMODE_MANUAL_AIRFLOW_LEVEL_SAF = 38
    REG_MAINBOARD_USERMODE_MANUAL_AIRFLOW_LEVEL_EAF = 39
    REG_MAINBOARD_FAN_MANUAL_STOP_ALLOWED = 40
    REG_MAINBOARD_SYSTEM_UNIT_MODEL1 = 41
    REG_MAINBOARD_SYSTEM_SERIAL_NUMBER1 = 42
    REG_MAINBOARD_SUW_REQUIRED = 43
    REG_MAINBOARD_FAN_REGULATION_UNIT = 44
    REG_MAINBOARD_SYSTEM_UNIT_FLOW = 45
    REG_MAINBOARD_SYSTEM_UNIT_PRESSURE = 46
    REG_MAINBOARD_FAN_REGULATION_PBAND = 47
    REG_MAINBOARD_FAN_REGULATION_ITIME = 48
    REG_MAINBOARD_K_FACTOR_SAF = 49
    REG_MAINBOARD_K_FACTOR_EAF = 50
    REG_MAINBOARD_WS_FAN_LEVEL_SCHEDULED = 51
    REG_MAINBOARD_WS_FAN_LEVEL_UNSCHEDULED = 52
    REG_MAINBOARD_SENSOR_SAT = 53
    REG_MAINBOARD_SENSOR_OAT = 54 # Sensor Outside air temp (Celsius * 10)
    REG_MAINBOARD_SENSOR_FPT = 55
    REG_MAINBOARD_SENSOR_RAT = 56
    REG_MAINBOARD_SENSOR_EAT = 57
    REG_MAINBOARD_SENSOR_ECT = 58
    REG_MAINBOARD_SENSOR_EFT = 59
    REG_MAINBOARD_SENSOR_OHT = 60
    REG_MAINBOARD_SENSOR_RHS = 61
    REG_MAINBOARD_SENSOR_BYS = 62
    REG_MAINBOARD_SENSOR_EMT = 63
    REG_MAINBOARD_SENSOR_RGS = 64
    REG_MAINBOARD_SENSOR_CO2S_1 = 65
    REG_MAINBOARD_SENSOR_CO2S_2 = 66
    REG_MAINBOARD_SENSOR_CO2S_3 = 67
    REG_MAINBOARD_SENSOR_CO2S_4 = 68
    REG_MAINBOARD_SENSOR_CO2S_5 = 69
    REG_MAINBOARD_SENSOR_CO2S_6 = 70
    REG_MAINBOARD_SENSOR_RHS_1 = 71
    REG_MAINBOARD_SENSOR_RHS_2 = 72
    REG_MAINBOARD_SENSOR_RHS_3 = 73
    REG_MAINBOARD_SENSOR_RHS_4 = 74
    REG_MAINBOARD_SENSOR_RHS_5 = 75
    REG_MAINBOARD_SENSOR_RHS_6 = 76
    REG_MAINBOARD_SENSOR_CO2S = 77
    REG_MAINBOARD_SENSOR_RHS_PDM = 78
    REG_MAINBOARD_SENSOR_P_SAF = 79
    REG_MAINBOARD_SENSOR_P_EAF = 80
    REG_MAINBOARD_SENSOR_FLOW_SAF = 81
    REG_MAINBOARD_SENSOR_FLOW_EAF = 82
    REG_MAINBOARD_SENSOR_RPM_SAF = 83
    REG_MAINBOARD_SENSOR_RPM_EAF = 84
    REG_MAINBOARD_SENSOR_FLOW_PIGGYBACK_SAF = 85
    REG_MAINBOARD_SENSOR_FLOW_PIGGYBACK_EAF = 86
    REG_MAINBOARD_SENSOR_DI_BYF = 87
    REG_MAINBOARD_SENSOR_PDM_EAT_CONFIGURED = 88
    REG_MAINBOARD_SENSOR_PDM_EAT_VALUE = 89
    REG_MAINBOARD_INPUT_EXTERNAL_CTRL_SAF = 90
    REG_MAINBOARD_INPUT_EXTERNAL_CTRL_EAF = 91
    REG_MAINBOARD_PDM_CONNECTED_RH = 92
    REG_MAINBOARD_PDM_CONNECTED_T = 93
    REG_MAINBOARD_PDM_CORRECTION_RH = 94
    REG_MAINBOARD_PDM_CORRECTION_T = 95
    REG_MAINBOARD_PIGGYBACK_1_PRESSURE_SAF = 96
    REG_MAINBOARD_PIGGYBACK_1_PRESSURE_EAF = 97
    REG_MAINBOARD_PIGGYBACK_1_SAF_COMPENSATION = 98
    REG_MAINBOARD_PIGGYBACK_1_EAF_COMPENSATION = 99
    REG_MAINBOARD_PIGGYBACK_1_MODE = 100
    REG_MAINBOARD_PIGGYBACK_2_MODE = 101
    REG_MAINBOARD_FUNCTION_ACTIVE_COOLING = 102
    REG_MAINBOARD_FUNCTION_ACTIVE_FREE_COOLING = 103
    REG_MAINBOARD_FUNCTION_ACTIVE_HEATING = 104
    REG_MAINBOARD_FUNCTION_ACTIVE_DEFROSTING = 105
    REG_MAINBOARD_FUNCTION_ACTIVE_HEAT_RECOVERY = 106
    REG_MAINBOARD_FUNCTION_ACTIVE_COOLING_RECOVERY = 107
    REG_MAINBOARD_FUNCTION_ACTIVE_MOISTURE_TRANSFER = 108
    REG_MAINBOARD_FUNCTION_ACTIVE_SECONDARY_AIR = 109
    REG_MAINBOARD_FUNCTION_ACTIVE_VACUUM_CLEANER = 110
    REG_MAINBOARD_FUNCTION_ACTIVE_COOKER_HOOD = 111
    REG_MAINBOARD_FUNCTION_ACTIVE_USER_LOCK = 112
    REG_MAINBOARD_FUNCTION_ACTIVE_ECO_MODE = 113
    REG_MAINBOARD_FUNCTION_ACTIVE_HEATER_COOL_DOWN = 114
    REG_MAINBOARD_FUNCTION_ACTIVE_PRESSURE_GUARD = 115
    REG_MAINBOARD_FUNCTION_ACTIVE_CDI_1 = 116
    REG_MAINBOARD_FUNCTION_ACTIVE_CDI_2 = 117
    REG_MAINBOARD_FUNCTION_ACTIVE_CDI_3 = 118
    REG_MAINBOARD_ALARM_TYPE_A = 119
    REG_MAINBOARD_ALARM_TYPE_B = 120
    REG_MAINBOARD_ALARM_TYPE_C = 121
    REG_MAINBOARD_ALARM_SAF_CTRL_ERROR = 122
    REG_MAINBOARD_ALARM_SAF_CTRL_ALARM = 123
    REG_MAINBOARD_ALARM_SAF_CTRL_CLEAR_ALARM = 124
    REG_MAINBOARD_ALARM_EAF_CTRL_ERROR = 125
    REG_MAINBOARD_ALARM_EAF_CTRL_ALARM = 126
    REG_MAINBOARD_ALARM_EAF_CTRL_CLEAR_ALARM = 127
    REG_MAINBOARD_ALARM_FROST_PROT_ERROR = 128
    REG_MAINBOARD_ALARM_FROST_PROT_ALARM = 129
    REG_MAINBOARD_ALARM_FROST_PROT_CLEAR_ALARM = 130
    REG_MAINBOARD_ALARM_DEFROSTING_ERROR = 131
    REG_MAINBOARD_ALARM_DEFROSTING_ALARM = 132
    REG_MAINBOARD_ALARM_DEFROSTING_CLEAR_ALARM = 133
    REG_MAINBOARD_ALARM_SAF_RPM_ERROR = 134
    REG_MAINBOARD_ALARM_SAF_RPM_ALARM = 135
    REG_MAINBOARD_ALARM_SAF_RPM_CLEAR_ALARM = 136
    REG_MAINBOARD_ALARM_EAF_RPM_ERROR = 137
    REG_MAINBOARD_ALARM_EAF_RPM_ALARM = 138
    REG_MAINBOARD_ALARM_EAF_RPM_CLEAR_ALARM = 139
    REG_MAINBOARD_ALARM_FPT_ERROR = 140
    REG_MAINBOARD_ALARM_FPT_ALARM = 141
    REG_MAINBOARD_ALARM_FPT_CLEAR_ALARM = 142
    REG_MAINBOARD_ALARM_OAT_ERROR = 143
    REG_MAINBOARD_ALARM_OAT_ALARM = 144
    REG_MAINBOARD_ALARM_OAT_CLEAR_ALARM = 145
    REG_MAINBOARD_ALARM_SAT_ERROR = 146
    REG_MAINBOARD_ALARM_SAT_ALARM = 147
    REG_MAINBOARD_ALARM_SAT_CLEAR_ALARM = 148
    REG_MAINBOARD_ALARM_RAT_ERROR = 149
    REG_MAINBOARD_ALARM_RAT_ALARM = 150
    REG_MAINBOARD_ALARM_RAT_CLEAR_ALARM = 151
    REG_MAINBOARD_ALARM_EAT_ERROR = 152
    REG_MAINBOARD_ALARM_EAT_ALARM = 153
    REG_MAINBOARD_ALARM_EAT_CLEAR_ALARM = 154
    REG_MAINBOARD_ALARM_ECT_ERROR = 155
    REG_MAINBOARD_ALARM_ECT_ALARM = 156
    REG_MAINBOARD_ALARM_ECT_CLEAR_ALARM = 157
    REG_MAINBOARD_ALARM_EFT_ERROR = 158
    REG_MAINBOARD_ALARM_EFT_ALARM = 159
    REG_MAINBOARD_ALARM_EFT_CLEAR_ALARM = 160
    REG_MAINBOARD_ALARM_OHT_ERROR = 161
    REG_MAINBOARD_ALARM_OHT_ALARM = 162
    REG_MAINBOARD_ALARM_OHT_CLEAR_ALARM = 163
    REG_MAINBOARD_ALARM_EMT_ERROR = 164
    REG_MAINBOARD_ALARM_EMT_ALARM = 165
    REG_MAINBOARD_ALARM_EMT_CLEAR_ALARM = 166
    REG_MAINBOARD_ALARM_RGS_ERROR = 167
    REG_MAINBOARD_ALARM_RGS_ALARM = 168
    REG_MAINBOARD_ALARM_RGS_CLEAR_ALARM = 169
    REG_MAINBOARD_ALARM_BYS_ERROR = 170
    REG_MAINBOARD_ALARM_BYS_ALARM = 171
    REG_MAINBOARD_ALARM_BYS_CLEAR_ALARM = 172
    REG_MAINBOARD_ALARM_SECONDARY_AIR_ERROR = 173
    REG_MAINBOARD_ALARM_SECONDARY_AIR_ALARM = 174
    REG_MAINBOARD_ALARM_SECONDARY_AIR_CLEAR_ALARM = 175
    REG_MAINBOARD_ALARM_FILTER_ERROR = 176
    REG_MAINBOARD_ALARM_FILTER_ALARM = 177
    REG_MAINBOARD_ALARM_FILTER_CLEAR_ALARM = 178
    REG_MAINBOARD_ALARM_EXTRA_CONTROLLER_ERROR = 179
    REG_MAINBOARD_ALARM_EXTRA_CONTROLLER_ALARM = 180
    REG_MAINBOARD_ALARM_EXTRA_CONTROLLER_CLEAR_ALARM = 181
    REG_MAINBOARD_ALARM_EXTERNAL_STOP_ERROR = 182
    REG_MAINBOARD_ALARM_EXTERNAL_STOP_ALARM = 183
    REG_MAINBOARD_ALARM_EXTERNAL_STOP_CLEAR_ALARM = 184
    REG_MAINBOARD_ALARM_MANUAL_OVERRIDE_OUTPUTS_ERROR = 185
    REG_MAINBOARD_ALARM_MANUAL_OVERRIDE_OUTPUTS_ALARM = 186
    REG_MAINBOARD_ALARM_MANUAL_OVERRIDE_OUTPUTS_CLEAR_ALARM = 187
    REG_MAINBOARD_ALARM_RH_ERROR = 188
    REG_MAINBOARD_ALARM_RH_ALARM = 189
    REG_MAINBOARD_ALARM_RH_CLEAR_ALARM = 190
    REG_MAINBOARD_ALARM_CO2_ERROR = 191
    REG_MAINBOARD_ALARM_CO2_ALARM = 192
    REG_MAINBOARD_ALARM_CO2_CLEAR_ALARM = 193
    REG_MAINBOARD_ALARM_LOW_SAT_ERROR = 194
    REG_MAINBOARD_ALARM_LOW_SAT_ALARM = 195
    REG_MAINBOARD_ALARM_LOW_SAT_CLEAR_ALARM = 196
    REG_MAINBOARD_ALARM_BYF_ERROR = 197
    REG_MAINBOARD_ALARM_BYF_ALARM = 198
    REG_MAINBOARD_ALARM_BYF_CLEAR_ALARM = 199
    REG_MAINBOARD_ALARM_PDM_RHS_ERROR = 200
    REG_MAINBOARD_ALARM_PDM_RHS_ALARM = 201
    REG_MAINBOARD_ALARM_PDM_RHS_CLEAR_ALARM = 202
    REG_MAINBOARD_ALARM_PDM_EAT_ERROR = 203
    REG_MAINBOARD_ALARM_PDM_EAT_ALARM = 204
    REG_MAINBOARD_ALARM_PDM_EAT_CLEAR_ALARM = 205
    REG_MAINBOARD_ALARM_MANUAL_FAN_STOP_ERROR = 206
    REG_MAINBOARD_ALARM_MANUAL_FAN_STOP_ALARM = 207
    REG_MAINBOARD_ALARM_MANUAL_FAN_STOP_CLEAR_ALARM = 208
    REG_MAINBOARD_ALARM_OVERHEAT_TEMPERATURE_ERROR = 209
    REG_MAINBOARD_ALARM_OVERHEAT_TEMPERATURE_ALARM = 210
    REG_MAINBOARD_ALARM_OVERHEAT_TEMPERATURE_CLEAR_ALARM = 211
    REG_MAINBOARD_ALARM_FIRE_ALARM_ERROR = 212
    REG_MAINBOARD_ALARM_FIRE_ALARM_ALARM = 213
    REG_MAINBOARD_ALARM_FIRE_ALARM_CLEAR_ALARM = 214
    REG_MAINBOARD_ALARM_FILTER_WARNING_ERROR = 215
    REG_MAINBOARD_ALARM_FILTER_WARNING_ALARM = 216
    REG_MAINBOARD_ALARM_FILTER_WARNING_CLEAR_ALARM = 217
    REG_MAINBOARD_ALARM_SAF_CTRL_TIMESTAMP_L = 218
    REG_MAINBOARD_ALARM_EAF_CTRL_TIMESTAMP_L = 219
    REG_MAINBOARD_ALARM_FROST_PROT_TIMESTAMP_L = 220
    REG_MAINBOARD_ALARM_DEFROSTING_TIMESTAMP_L = 221
    REG_MAINBOARD_ALARM_SAF_RPM_TIMESTAMP_L = 222
    REG_MAINBOARD_ALARM_EAF_RPM_TIMESTAMP_L = 223
    REG_MAINBOARD_ALARM_FPT_TIMESTAMP_L = 224
    REG_MAINBOARD_ALARM_OAT_TIMESTAMP_L = 225
    REG_MAINBOARD_ALARM_SAT_TIMESTAMP_L = 226
    REG_MAINBOARD_ALARM_RAT_TIMESTAMP_L = 227
    REG_MAINBOARD_ALARM_EAT_TIMESTAMP_L = 228
    REG_MAINBOARD_ALARM_ECT_TIMESTAMP_L = 229
    REG_MAINBOARD_ALARM_EFT_TIMESTAMP_L = 230
    REG_MAINBOARD_ALARM_OHT_TIMESTAMP_L = 231
    REG_MAINBOARD_ALARM_EMT_TIMESTAMP_L = 232
    REG_MAINBOARD_ALARM_RGS_TIMESTAMP_L = 233
    REG_MAINBOARD_ALARM_BYS_TIMESTAMP_L = 234
    REG_MAINBOARD_ALARM_SECONDARY_AIR_TIMESTAMP_L = 235
    REG_MAINBOARD_ALARM_FILTER_TIMESTAMP_L = 236
    REG_MAINBOARD_ALARM_EXTRA_CONTROLLER_TIMESTAMP_L = 237
    REG_MAINBOARD_ALARM_EXTERNAL_STOP_TIMESTAMP_L = 238
    REG_MAINBOARD_ALARM_RH_TIMESTAMP_L = 239
    REG_MAINBOARD_ALARM_CO2_TIMESTAMP_L = 240
    REG_MAINBOARD_ALARM_LOW_SAT_TIMESTAMP_L = 241
    REG_MAINBOARD_ALARM_BYF_TIMESTAMP_L = 242
    REG_MAINBOARD_ALARM_PDM_RHS_TIMESTAMP_L = 243
    REG_MAINBOARD_ALARM_PDM_EAT_TIMESTAMP_L = 244
    REG_MAINBOARD_ALARM_MANUAL_OVERRIDE_OUTPUTS_TIMESTAMP_L = 245
    REG_MAINBOARD_ALARM_MANUAL_FAN_STOP_TIMESTAMP_L = 246
    REG_MAINBOARD_ALARM_OVERHEAT_TEMPERATURE_TIMESTAMP_L = 247
    REG_MAINBOARD_ALARM_FIRE_ALARM_TIMESTAMP_L = 248
    REG_MAINBOARD_ALARM_FILTER_WARNING_TIMESTAMP_L = 249
    REG_MAINBOARD_USERMODE_REMAINING_TIME_L = 250
    REG_MAINBOARD_USERMODE_HOLIDAY_TIME = 251
    REG_MAINBOARD_USERMODE_AWAY_TIME = 252
    REG_MAINBOARD_USERMODE_FIREPLACE_TIME = 253
    REG_MAINBOARD_USERMODE_REFRESH_TIME = 254
    REG_MAINBOARD_USERMODE_CROWDED_TIME = 255
    REG_MAINBOARD_DEMC_CO2_SETTINGS_ON_OFF = 256
    REG_MAINBOARD_DEMC_RH_SETTINGS_ON_OFF = 257
    REG_MAINBOARD_WS_ANY_DEFINED = 258
    REG_MAINBOARD_HEAT_EXCHANGER_TYPE = 259
    REG_MAINBOARD_UNIT_CONFIG_REHEATER_TYPE = 260
    REG_MAINBOARD_UNIT_CONFIG_BYPASS_LOCATION = 261
    REG_MAINBOARD_DEFROSTING_DISABLE = 262
    REG_MAINBOARD_CFG_HEAT_EXCHANGER_ACTUATOR_TYPE = 263
    REG_MAINBOARD_PASSIVE_HOUSE_ACTIVATION = 264
    REG_MAINBOARD_PASSIVE_HOUSE_CERTIFICATION = 265
    REG_MAINBOARD_PASSIVE_HOUSE_MAX_LIMIT = 266
    REG_MAINBOARD_CFG_HEATER_ACTUATOR_TYPE = 267
    REG_MAINBOARD_CFG_CHANGE_OVER_ACTUATOR_TYPE = 268
    REG_MAINBOARD_HEATER_CIRC_PUMP_START_T = 269
    REG_MAINBOARD_HEATER_CIRC_PUMP_STOP_DELAY = 270
    REG_MAINBOARD_CHANGE_OVER_CIRC_PUMP_START_T = 271
    REG_MAINBOARD_CHANGE_OVER_CIRC_PUMP_STOP_DELAY = 272
    REG_MAINBOARD_UNIT_CONFIG_COOLER = 273
    REG_MAINBOARD_CFG_COOLER_ACTUATOR_TYPE = 274
    REG_MAINBOARD_COOLER_OAT_INTERLOCK_T = 275
    REG_MAINBOARD_COOLER_CIRC_PUMP_STOP_DELAY = 276
    REG_MAINBOARD_UNIT_CONFIG_EXTRA_CONTROLLER = 277
    REG_MAINBOARD_EXTRA_CONTROLLER_PREHEATER_SETPOINT_TYPE = 278
    REG_MAINBOARD_EXTRA_CONTROLLER_SET_PI_SETPOINT = 279
    REG_MAINBOARD_EXTRA_CONTROLLER_SET_PI_PBAND = 280
    REG_MAINBOARD_EXTRA_CONTROLLER_SET_PI_ITIME = 281
    REG_MAINBOARD_CFG_EXTRA_ACTUATOR_TYPE = 282
    REG_MAINBOARD_EXTRA_CONTROLLER_CIRC_PUMP_START_T = 283
    REG_MAINBOARD_EXTRA_CONTROLLER_CIRC_PUMP_STOP_DELAY = 284
    REG_MAINBOARD_EXTRA_CONTROLLER_GEO_PRECOOLER_SP = 285
    REG_MAINBOARD_EXTRA_CONTROLLER_GEO_PRECOOLER_ACTIVATION_T = 286
    REG_MAINBOARD_EXTRA_CONTROLLER_GEO_PREHEATER_SP = 287
    REG_MAINBOARD_EXTRA_CONTROLLER_GEO_PREHEATER_ACTIVATION_T = 288
    REG_MAINBOARD_PASSWD_ADMIN = 289
    REG_MAINBOARD_PU_UPDATE_SOURCE = 290
    REG_MAINBOARD_PU_RUNNING_VERSION_MAJOR = 291
    REG_MAINBOARD_PU_RUNNING_VERSION_MINOR = 292
    REG_MAINBOARD_PU_RUNNING_VERSION_BUILD = 293

    # Add more constants here as needed

    @classmethod
    def get_register_name(cls, register_number: int) -> str:
        """Get the register name for a given register number."""
        for attr, value in cls.__dict__.items():
            if not attr.startswith("__") and value == register_number:
                return attr
        return f"UNKNOWN_REGISTER_{register_number}"


    @classmethod
    def get_register_name_by_number(cls, register_number: int) -> str:
        """Get the register name for a given register number."""
        for attr, value in cls.__dict__.items():
            if not attr.startswith("__") and value == register_number:
                return attr
        return f"UNKNOWN_REGISTER_{register_number}"


    @classmethod
    def get_register_name_without_prefix(cls, register_number: int) -> str:
        """Get the register name without the REG_MAINBOARD_ prefix."""
        full_name = cls.get_register_name_by_number(register_number)
        return full_name.replace("REG_MAINBOARD_", "")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Debug script for Python 2.7 compatibility checker

This script helps diagnose issues when trying to run the SystemAIR-API with Python 2.7.
The SystemAIR-API library requires Python 3.7+ and won't work with Python 2.
This diagnostic tool:

1. Checks your Python version
2. Examines your environment setup
3. Verifies project structure and dependencies
4. Provides guidance on how to set up a Python 3 environment

Usage:
    python debug_api_py2.py
"""

from __future__ import print_function
import os
import sys
import json
import traceback
import platform

def main():
    """Main diagnostic function."""
    log_file = open("debug_py2.log", "w")
    
    def log(msg):
        """Log to both stdout and file."""
        print(msg)
        log_file.write(msg + "\n")
        log_file.flush()
    
    log("SystemAIR API Compatibility Check")
    log("=" * 50)
    
    # Check Python version
    python_version = ".".join(map(str, sys.version_info[:3]))
    log("Python version: " + python_version)
    log("Python executable: " + sys.executable)
    log("Platform: " + platform.platform())
    
    is_python3 = sys.version_info[0] >= 3
    if is_python3:
        log("\nYou are using Python 3, which is compatible with SystemAIR-API.")
        if sys.version_info[1] < 7:
            log("However, Python 3.7+ is recommended. You're using " + python_version)
    else:
        log("\nWARNING: You are using Python 2, which is NOT compatible with SystemAIR-API.")
        log("SystemAIR-API requires Python 3.7 or higher.")
    
    # Check if we have Python 3 available
    log("\nChecking for Python 3 availability...")
    python3_paths = [
        "/usr/bin/python3",
        "/usr/local/bin/python3",
        "C:\\Python39\\python.exe",
        "C:\\Python38\\python.exe",
        "C:\\Python37\\python.exe"
    ]
    
    python3_found = False
    for path in python3_paths:
        if os.path.exists(path):
            log("+ Found Python 3 at: " + path)
            python3_found = True
            break
    
    if not python3_found:
        log("x No common Python 3 installation found")
        log("  You'll need to install Python 3.7 or later")
    
    # Check if .env file exists
    env_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), ".env")
    log("\nChecking for .env file: " + env_file)
    if os.path.exists(env_file):
        log("+ .env file exists")
        with open(env_file, "r") as f:
            log("  First few lines (without passwords):")
            for line in f.readlines()[:5]:
                if "PASSWORD" not in line.upper():
                    log("  " + line.strip())
    else:
        log("x .env file does NOT exist")
        log("  You should create one with your Systemair credentials:")
        log("  EMAIL=your.email@example.com")
        log("  PASSWORD=your_password")
    
    # Check Python packages availability
    log("\nChecking for required packages:")
    required_packages = [
        ("requests", "For API communication"),
        ("websocket-client", "For real-time updates"),
        ("bs4", "For authentication form parsing"),
        ("dotenv", "For loading environment variables"),
        ("textual", "For the TUI interface")
    ]
    
    for package, purpose in required_packages:
        try:
            # Try to import the package
            __import__(package)
            log("+ " + package + " package is installed - " + purpose)
        except ImportError:
            log("x " + package + " package is NOT installed - " + purpose)
    
    # Check project structure
    log("\nChecking project structure:")
    project_dir = os.path.dirname(os.path.abspath(__file__))
    log("Project directory: " + project_dir)
    
    # Check key files
    key_files = [
        "systemair_api/auth/authenticator.py",
        "systemair_api/api/systemair_api.py",
        "systemair_api/models/ventilation_unit.py",
        "systemair_api/api/websocket_client.py",
        "examples/systemair_tui.py"
    ]
    
    all_files_exist = True
    for file_path in key_files:
        full_path = os.path.join(project_dir, file_path)
        if os.path.exists(full_path):
            log("+ " + file_path + " exists")
            # Try to detect Python 3 features
            with open(full_path, "r") as f:
                try:
                    content = f.read()
                    # Check for type annotations (Python 3 feature)
                    if ": " in content and "->" in content:
                        log("  Contains Python 3 type annotations")
                except:
                    log("  Error reading file")
        else:
            log("x " + file_path + " does NOT exist")
            all_files_exist = False
    
    # Check for textual CSS file
    tui_css_path = os.path.join(project_dir, "examples", "systemair_tui.css")
    if os.path.exists(tui_css_path):
        log("+ TUI CSS file exists")
    else:
        log("x TUI CSS file does NOT exist - it will be created on first run")
    
    # Recommendations section
    log("\n" + "=" * 50)
    log("DIAGNOSIS:")
    
    if not is_python3:
        log("The SystemAIR-API library requires Python 3.7+")
        log("You are currently using Python " + python_version + " which is not compatible.")
    
    if not all_files_exist:
        log("Some essential project files are missing. Ensure you have downloaded the complete project.")
    
    log("\nRECOMMENDATIONS:")
    if not is_python3:
        log("1. Install Python 3.7 or later from https://www.python.org/downloads/")
        log("2. Create a Python 3 virtual environment:")
        log("   python3 -m venv .venv")
        log("3. Activate the virtual environment:")
        if platform.system() == "Windows":
            log("   .venv\\Scripts\\activate")
        else:
            log("   source .venv/bin/activate")
        log("4. Install required packages:")
        log("   pip install -r requirements.txt")
        log("   pip install -r examples/requirements-tui.txt  # For TUI")
    else:
        # Python 3 is already being used
        log("1. Create a virtual environment if you haven't already:")
        log("   python -m venv .venv")
        log("2. Activate the virtual environment:")
        if platform.system() == "Windows":
            log("   .venv\\Scripts\\activate")
        else:
            log("   source .venv/bin/activate")
        log("3. Install required packages:")
        log("   pip install -r requirements.txt")
        log("   pip install -r examples/requirements-tui.txt  # For TUI")
    
    log("\nTroubleshooting the TUI:")
    log("1. Ensure textual is installed: pip install 'textual>=0.27.0'")
    log("2. Run the diagnostic script for API connectivity: python debug_api.py")
    log("3. Check the log files: systemair_tui_debug.log and debug_api.log")
    log("4. If units don't appear, verify API connectivity and credentials")
    
    log_file.close()
    log("\nDebug information written to debug_py2.log")
    
    # Return code: 0 for Python 3, 1 for Python 2
    return 0 if is_python3 else 1

if __name__ == "__main__":
    exit_code = main()
    sys.exit(exit_code)


"""SystemairAPI - Core API communication module for Systemair ventilation units."""

from typing import Dict, List, Optional, Any, Union, cast
import requests
from systemair_api.utils.constants import APIEndpoints
from systemair_api.utils.exceptions import APIError, DeviceNotFoundError, RateLimitError

class SystemairAPI:
    """Core API interface for communicating with Systemair Home Solutions API.
    
    Provides methods for discovering devices, fetching device status,
    and sending control commands to ventilation units.
    """
    
    def __init__(self, access_token: str) -> None:
        """Initialize the SystemairAPI with an access token.
        
        Args:
            access_token: A valid JWT access token from authentication
        """
        self.access_token: str = access_token
        self.headers: Dict[str, str] = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:128.0) Gecko/20100101 Firefox/128.0',
            'Accept': '*/*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Referer': 'https://homesolutions.systemair.com/device/home',
            'content-type': 'application/json',
            'x-access-token': self.access_token,
            'Origin': 'https://homesolutions.systemair.com',
            'Connection': 'keep-alive',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
        }
        
    def update_token(self, access_token: str) -> None:
        """Update the access token used for API requests.
        
        This method should be called when the token is refreshed.
        
        Args:
            access_token: The new access token
        """
        self.access_token = access_token
        self.headers['x-access-token'] = access_token

    def broadcast_device_statuses(self, device_ids: List[str]) -> Dict[str, Any]:
        """Broadcast requests for device statuses to trigger WebSocket updates.
        
        Args:
            device_ids: List of device identifiers to request updates for
            
        Returns:
            dict: API response
            
        Raises:
            APIError: If the API request fails
            DeviceNotFoundError: If one or more devices are not found
            RateLimitError: If rate limit is exceeded
        """
        data = {
            "variables": {"deviceIds": device_ids},
            "query": """
            query ($deviceIds: [String]!) {
                BroadcastDeviceStatuses(deviceIds: $deviceIds)
            }
            """
        }

        try:
            response = requests.post(APIEndpoints.GATEWAY, headers=self.headers, json=data)
            
            if response.status_code == 429:
                retry_after = response.headers.get('Retry-After')
                retry_seconds = int(retry_after) if retry_after and retry_after.isdigit() else None
                raise RateLimitError(retry_after=retry_seconds)
                
            response.raise_for_status()
            result = cast(Dict[str, Any], response.json())
            
            # Check for error in response data
            if 'errors' in result:
                errors = result['errors']
                for error in errors:
                    if 'message' in error and any(device_id in error.get('message', '') for device_id in device_ids):
                        raise DeviceNotFoundError(device_ids[0], error.get('message'))
                raise APIError(message=errors[0].get('message', 'Unknown API error'), 
                              response_data=result)
                              
            return result
        except requests.exceptions.RequestException as e:
            raise APIError(f"Failed to broadcast device statuses: {str(e)}", 
                          getattr(e, 'response', None) and getattr(e.response, 'status_code', None))

    def fetch_device_status(self, device_id: str) -> Dict[str, Any]:
        """Fetch detailed status for a specific device.
        
        Args:
            device_id: The unique identifier of the device
            
        Returns:
            dict: API response with detailed device status
            
        Raises:
            APIError: If the API request fails
            DeviceNotFoundError: If the device is not found
            RateLimitError: If rate limit is exceeded
        """
        headers = self.headers.copy()
        headers['device-id'] = device_id
        headers['device-type'] = 'LEGACY'

        data = {
            "variables": {"input": {"route": "/home", "viewId": ""}},
            "query": """
            query ($input: GetViewInputType!) {
                GetView(input: $input) {
                    children {
                        type
                        properties
                    }
                }
            }
            """
        }

        try:
            response = requests.post(APIEndpoints.REMOTE, headers=headers, json=data)
            
            if response.status_code == 404:
                raise DeviceNotFoundError(device_id)
                
            if response.status_code == 429:
                retry_after = response.headers.get('Retry-After')
                retry_seconds = int(retry_after) if retry_after and retry_after.isdigit() else None
                raise RateLimitError(retry_after=retry_seconds)
                
            response.raise_for_status()
            result = cast(Dict[str, Any], response.json())
            
            # Check for error in response data
            if 'errors' in result:
                errors = result['errors']
                for error in errors:
                    if 'message' in error and device_id in error.get('message', ''):
                        raise DeviceNotFoundError(device_id, error.get('message'))
                raise APIError(message=errors[0].get('message', 'Unknown API error'), 
                              response_data=result)
                              
            return result
        except requests.exceptions.RequestException as e:
            if getattr(e, 'response', None) and getattr(e.response, 'status_code', None) == 404:
                raise DeviceNotFoundError(device_id, str(e))
            raise APIError(f"Failed to fetch device status: {str(e)}", 
                          getattr(e, 'response', None) and getattr(e.response, 'status_code', None))

    def get_account_devices(self) -> Dict[str, Any]:
        """Get all devices associated with the current account.
        
        Returns:
            dict: API response with devices list
            
        Raises:
            APIError: If the API request fails
            RateLimitError: If rate limit is exceeded
        """
        data = {
            "operationName": "GetLoggedInAccount",
            "variables": {},
            "query": """
            query GetLoggedInAccount {
              GetAccountDevices {
                identifier
                name
                street
                zipcode
                city
                country
                deviceType {
                  entry
                  module
                  scope
                  type
                }
              }
            }
            """
        }

        try:
            response = requests.post(APIEndpoints.GATEWAY, headers=self.headers, json=data)
            
            if response.status_code == 429:
                retry_after = response.headers.get('Retry-After')
                retry_seconds = int(retry_after) if retry_after and retry_after.isdigit() else None
                raise RateLimitError(retry_after=retry_seconds)
                
            response.raise_for_status()
            result = cast(Dict[str, Any], response.json())
            
            # Check for error in response data
            if 'errors' in result:
                errors = result['errors']
                raise APIError(message=errors[0].get('message', 'Unknown API error'), 
                              response_data=result, 
                              status_code=response.status_code)
                              
            return result
        except requests.exceptions.RequestException as e:
            raise APIError(f"Failed to fetch account devices: {str(e)}", 
                          getattr(e, 'response', None) and getattr(e.response, 'status_code', None))


    def write_data_item(self, device_id: str, register_id: int, value: Union[int, float, str]) -> Dict[str, Any]:
        """Write a value to a specific register on a device.
        
        Args:
            device_id: The unique identifier of the device
            register_id: The register ID to write to
            value: The value to write
            
        Returns:
            dict: API response indicating success
            
        Raises:
            APIError: If the API request fails
            DeviceNotFoundError: If the device is not found
            RateLimitError: If rate limit is exceeded
            ValidationError: If the provided value is invalid
        """
        headers = self.headers.copy()
        headers['device-id'] = device_id
        headers['device-type'] = 'LEGACY'

        data = {
            "variables": {
                "input": {
                    "dataPoints": [{"id": register_id, "value": str(value)}]
                }
            },
            "query": """
            mutation ($input: WriteDataItemsInput!) {
                WriteDataItems(input: $input)
            }
            """
        }

        try:
            response = requests.post(APIEndpoints.REMOTE, headers=headers, json=data)
            
            if response.status_code == 404:
                raise DeviceNotFoundError(device_id)
                
            if response.status_code == 429:
                retry_after = response.headers.get('Retry-After')
                retry_seconds = int(retry_after) if retry_after and retry_after.isdigit() else None
                raise RateLimitError(retry_after=retry_seconds)
                
            response.raise_for_status()
            result = cast(Dict[str, Any], response.json())
            
            # Check for error in response data
            if 'errors' in result:
                errors = result['errors']
                for error in errors:
                    msg = error.get('message', '')
                    if 'device' in msg.lower() and 'not found' in msg.lower():
                        raise DeviceNotFoundError(device_id, msg)
                    if 'invalid' in msg.lower() or 'validation' in msg.lower():
                        from systemair_api.utils.exceptions import ValidationError
                        raise ValidationError(message=msg, field=f"register_{register_id}", value=value)
                raise APIError(message=errors[0].get('message', 'Unknown API error'), 
                              response_data=result)
                              
            return result
        except requests.exceptions.RequestException as e:
            if getattr(e, 'response', None) and getattr(e.response, 'status_code', None) == 404:
                raise DeviceNotFoundError(device_id, str(e))
            raise APIError(f"Failed to write data to device: {str(e)}", 
                          getattr(e, 'response', None) and getattr(e.response, 'status_code', None))


from setuptools import setup, find_packages
import os
import re


# Read version from __version__.py
def get_version():
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, 'systemair_api', '__version__.py'), encoding='utf-8') as f:
        version_file = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


# Read the contents of the README file
with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="systemair_api",
    version=get_version(),
    description="Python library for communicating with and controlling Systemair ventilation units",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Henning Berge",
    author_email="henning.ber@gmail.com",
    url="https://github.com/Promises/SystemAIR-API",
    project_urls={
        "Bug Tracker": "https://github.com/Promises/SystemAIR-API/issues",
        "Documentation": "https://github.com/Promises/SystemAIR-API#readme",
        "Source Code": "https://github.com/Promises/SystemAIR-API",
    },
    packages=find_packages(exclude=["tests", "notes"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests",
        "websocket-client",
        "beautifulsoup4",
        "python-dotenv",
    ],
    extras_require={
        "dev": [
            "pytest",
            "pytest-cov",
            "pytest-mock",
            "requests-mock",
            "black",
            "isort",
            "flake8",
            "mypy",
            "sphinx",
            "sphinx-rtd-theme",
            "sphinx-autodoc-typehints",
            "pre-commit",
            "build",
            "twine",
        ],
    },
)


"""SystemairWebSocket - WebSocket client for real-time updates from Systemair ventilation units."""

import websocket
import json
import threading
import ssl
from typing import Any, Callable, Dict, Optional, Union, cast
from websocket import WebSocket, WebSocketApp

class SystemairWebSocket:
    """WebSocket client for real-time updates from Systemair Home Solutions API.
    
    Establishes a persistent connection to the Systemair WebSocket server
    and processes incoming messages through a callback function.
    """
    
    def __init__(self, access_token: str, on_message_callback: Callable[[Dict[str, Any]], None]) -> None:
        """Initialize the WebSocket client.
        
        Args:
            access_token: A valid JWT access token from authentication
            on_message_callback: Callback function that will be called with message data
        """
        self.access_token: str = access_token
        self.on_message_callback: Callable[[Dict[str, Any]], None] = on_message_callback
        self.ws: Optional[WebSocketApp] = None
        self.thread: Optional[threading.Thread] = None

    def on_message(self, ws: WebSocket, message: Any) -> None:
        """Handle incoming WebSocket messages.
        
        Args:
            ws: WebSocket connection
            message: Raw message data
        """
        data = json.loads(message)
        self.on_message_callback(data)

    def on_error(self, ws: WebSocket, error: Any) -> None:
        """Handle WebSocket errors.
        
        Args:
            ws: WebSocket connection
            error: Error information
        """
        # Log to stdout for diagnostic purposes, but keep it minimal
        print(f"WebSocket error: {error}")

    def on_close(self, ws: WebSocket, close_status_code: Any, close_msg: Any) -> None:
        """Handle WebSocket connection closure.
        
        Args:
            ws: WebSocket connection
            close_status_code: Status code for the closure
            close_msg: Closure message
        """
        # Important status messages are kept to help diagnose issues
        if close_status_code:
            print(f"WebSocket connection closed with code: {close_status_code}")
        else:
            print("WebSocket connection closed")

    def on_open(self, ws: WebSocket) -> None:
        """Handle WebSocket connection opening.
        
        Args:
            ws: WebSocket connection
        """
        # Connection established notification is useful for debugging
        print("WebSocket connection opened")

    def connect(self) -> None:
        """Establish a WebSocket connection in a separate thread.
        
        The connection is opened in a daemon thread to allow the main program
        to continue execution.
        """
        # Disable WebSocket trace output to keep logs cleaner
        websocket.enableTrace(False)
        self.ws = websocket.WebSocketApp(
            f"wss://homesolutions.systemair.com/streaming/",
            header=[
                f"Sec-WebSocket-Protocol: accessToken, {self.access_token}",
                "Origin: https://homesolutions.systemair.com",
            ],
            on_open=self.on_open,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
        )
        self.thread = threading.Thread(target=self.ws.run_forever, kwargs={"sslopt": {"cert_reqs": ssl.CERT_NONE}})
        self.thread.daemon = True
        self.thread.start()

    def disconnect(self) -> None:
        """Close the WebSocket connection and clean up resources."""
        if self.ws:
            self.ws.close()
        if self.thread:
            self.thread.join()


"""SystemAIR-API - Python library for controlling Systemair ventilation units."""

from systemair_api.api.systemair_api import SystemairAPI
from systemair_api.auth.authenticator import SystemairAuthenticator
from systemair_api.models.ventilation_unit import VentilationUnit
from systemair_api.api.websocket_client import SystemairWebSocket
from systemair_api.utils.exceptions import (
    SystemairError,
    AuthenticationError,
    TokenRefreshError,
    APIError,
    DeviceNotFoundError,
    WebSocketError,
    RateLimitError,
    ValidationError
)
from systemair_api.utils.constants import UserModes
from systemair_api.__version__ import __version__

__all__ = [
    'SystemairAPI',
    'SystemairAuthenticator', 
    'VentilationUnit',
    'SystemairWebSocket',
    'SystemairError',
    'AuthenticationError',
    'TokenRefreshError',
    'APIError',
    'DeviceNotFoundError',
    'WebSocketError',
    'RateLimitError',
    'ValidationError',
    'UserModes',
    '__version__'
]